package com.example.demo.orders.service;

import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.repository.OrderRepository;

@Service
public class OrderService {
    private final OrderRepository repository;

    public OrderService(OrderRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<OrderEntity> getAll(Long userId, List<Long> lines) {
        if (userId != 0L && !lines.isEmpty()) {
            return repository.findByUserIdAndLinesProductIds(userId, lines);
        }
        if (userId == 0L && !lines.isEmpty()) {
            return repository.findByLinesProductIds(lines);
        }
        if (userId != 0L && lines.isEmpty()) {
            return repository.findByUserId(userId);
        }
        return repository.findAll();
    }

    @Transactional(readOnly = true)
    public OrderEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(OrderEntity.class, id));
    }

    public OrderEntity create(OrderEntity entity) {
        return repository.save(entity);
    }

    @Transactional
    public OrderEntity update(Long id, OrderEntity entity) {
        final OrderEntity existEntity = get(id);
        existEntity.setUser(entity.getUser());

        // Очищаем текущие строки заказа и добавляем новые
        existEntity.getLines().clear();
        for (OrderLineEntity newOrderLine : entity.getLines()) {
            existEntity.addOrderLine(newOrderLine);
        }

        return repository.save(existEntity);
    }

    @Transactional
    public OrderEntity delete(Long id) {
        final OrderEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
