package com.example.demo.products.service;

import java.util.List;

import org.springframework.stereotype.Service;
import java.util.stream.StreamSupport;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.products.model.ProductEntity;
import com.example.demo.products.repository.ProductRepository;

import org.springframework.transaction.annotation.Transactional;

@Service
public class ProductService {
    private final ProductRepository repository;

    public ProductService(ProductRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<ProductEntity> getAll(Long typeId) {
        if (typeId != null && typeId != 0L) {
            return repository.findByType_Id(typeId);
        } else {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
    }

    @Transactional(readOnly = true)
    public ProductEntity get(Long id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(ProductEntity.class, id));
    }

    @Transactional
    public ProductEntity create(ProductEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public ProductEntity update(Long id, ProductEntity entity) {
        final ProductEntity exisEntity = get(id);
        exisEntity.setName(entity.getName());
        return repository.save(exisEntity);
    }

    @Transactional
    public ProductEntity delete(Long id) {
        final ProductEntity existEntity = get(id);
        repository.delete(existEntity);
        return existEntity;
    }
}
