package com.example.demo.users.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;

@Service
public class UserService {
    private final UserRepository repository;

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    private void checkEmail(String email) {
        if (repository.findByEmailIgnoreCase(email).isPresent()) {
            throw new IllegalArgumentException(
                    String.format("User with email %s is already exists", email));
        }
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public UserEntity get(long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        checkEmail(entity.getEmail());
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(Long id, UserEntity entity) {
        checkEmail(entity.getEmail());
        final UserEntity existEntity = get(id);
        existEntity.setFullname(entity.getFullname());
        existEntity.setSurname(entity.getSurname());
        existEntity.setEmail(entity.getEmail());
        existEntity.setPassword(entity.getPassword());
        repository.save(existEntity);
        return existEntity;
    }

    @Transactional
    public UserEntity delete(Long id) {
        final UserEntity existEntity = get(id);
        repository.delete(existEntity);
        return existEntity;
    }
}
